@@ -1,27 +1,22 @@
-#ifndef U_FONTS_H
-#define U_FONTS_H
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1991 by Brian V. Smith
+ * Copyright (c) 1989-1998 by Brian V. Smith
  *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software subject to the restriction stated
- * below, and to permit persons who receive copies from any such party to
- * do so, with the only requirement being that this copyright notice remain
- * intact.
- * This license includes without limitation a license to do the foregoing
- * actions under any patents of the party supplying this software to the 
- * X Consortium.
+ * and/or sell copies of the Software, and to permit persons who receive
+ * copies from any such party to do so, with the only requirement being
+ * that this copyright notice remain intact.
  *
  */
 
-#define MIN_P_SIZE 6
-#define MAX_P_SIZE 30
+
+
+#ifndef U_FONTS_H
+#define U_FONTS_H
 
 #define DEF_FONTSIZE 12		/* default font size in pts */
 #define DEF_PS_FONT 0
@@ -32,6 +27,14 @@
 #define LATEX_FONTPANE_HT 20
 #define NUM_FONTS 35
 #define NUM_LATEX_FONTS 6
+/* font number for the "nil" font (when user wants tiny text) */
+#define NILL_FONT NUM_FONTS
+
+extern int		psfontnum();
+extern int		latexfontnum();
+extern struct _xfstruct	x_fontinfo[];
+extern struct _fstruct	ps_fontinfo[];
+extern struct _fstruct	latex_fontinfo[];
 
 /* element of linked list for each font
    The head of list is for the different font NAMES,
